#pragma CODE

#include "bindec.h"

// Module for converting between ascii null-terminated strings and binary values.


// Function 'bin2asc' converts an integer (-32768..32767) into
// an 6-byte ASCII-string, plus a 0-terminator.
//
// Format of string: <spaces(max 5)> <sign(only if negative)> <digits(at least 1)> <0>
//
// Parameters:
// 	in_bin  - 	binary value to be converted
//	out_asc - 	generic pointer to location where to
//			store the ASCII-string (reserve at least 7 bytes)
//				
void int2asc(int in_bin, char *out_asc) {
   byte i;
   bit sign = 0; // 1 if negative

   // Reset string with spaces and a null-terminator
   for (i=0; i<6; i++) *(out_asc++) = 0x20;	
   *out_asc = '\0';

   // If negative then get absolute value
   if ( in_bin < 0 ) {
     sign = 1;
     in_bin = -in_bin;
   }
   
   // Derive single digits, starting with least significant
   do {
      // Derive digit, convert into ASCII and store
      // backwards in output string
      *(--out_asc) = (unsigned)in_bin%10 + '0'; 
      in_bin = (unsigned)in_bin / (unsigned)10;  // Prepare for next digit
   } while (in_bin != 0); // Finish, when 0 reached
   if (sign) *(--out_asc) = '-';
}

// same as last, just with byte
void char2asc(byte in_bin, char *out_asc) {
   byte i;
   // Reset string with spaces and a null-terminator
   for (i=0; i<3; i++) *(out_asc++) = ' ';
   *out_asc = '\0';
   // Derive single digits, starting with least significant
   do {
      // Derive digit, convert into ASCII and store
      // backwards in output string
      *(--out_asc) = in_bin%10 + '0'; 
      in_bin = in_bin / 10;  // Prepare for next digit
   } while (in_bin != 0); // Finish, when 0 reached
}

 
// Function 'asc2bin' converts an ASCII-string into an signed int (-32767..32767)
//
// Format of string:
//
//  { <space> } [ <+/-> ] <ASCII-digits(max 5)> <0>
//
// The function returns FALSE (0), if format of the string is violated. 
// Otherwise TRUE (1) is returned.
//
// Parameters:
//	 in_asc  - generic pointer to string to be converted
//	 out_bin - generic pointer to an integer where to
//            place binary result
//
char asc2int(char *in_asc, int *out_bin) {
  byte dig = 0;
  bit sign = 0; // negative=1
  byte count = 0; // max 5 digits

  *out_bin=0;			      // Initialise binary value
  
  while (*in_asc++ == ' ');  // Skip leading spaces
  in_asc--;	              // in_asc points to first digit

  // Derive single digits, starting with most significant
  switch (*in_asc) {
    case '-':
      sign = 1; 
      in_asc++;
      break;
      
    case '+':
      sign = 0;
      in_asc++;
      break;
  }

  while ( (dig = *in_asc++) != '\0') {
	  dig = dig - '0';
     // Error if character is not a digit
     if (dig > 9) return FALSE;

     
     //Check if range is exceeded (range is 0..32767)
     count++;
     if ( count>5 ) return FALSE;
     if ( count==5 && 
         (*out_bin > 3276 || (*out_bin==3276 && dig > 7)) ) return FALSE;

     //Convert ASCII-digit to decimal and calculate binary value
     *out_bin = *out_bin * 10 + dig;

  }

  if (sign) *out_bin = -*out_bin;
  
  return TRUE;

}

