#pragma CODE
#pragma NOMOD517

#include "general.h"
#include "bindec.h"
#include "lcd.h"
#include "dac.h"
#include "keyboard.h"
#include "fp.h"

#define RL_POWER P4_2
#define RL_NEAR P4_3
#define RL_FAR P4_4

// current time variables
static xdata int msec=0; // counts half millisecs!!!
static xdata char sec=0, min=0, hour=0, day=0;

// scaling factors for the system
static xdata struct fp scale1 = { BIAS+2, 0, 0x9ac6 };
static xdata struct fp scale2 = { BIAS+5, 0, 0x876D };

// status for breakers
static bit power=0, near=0, far=0;

// current R and L in floating-point
static xdata int R, L;
static xdata int Rr, Ll; // temps

// Resistance limits for Short-Circuit condition
static xdata char far_limit = 10;
static xdata char near_limit = 6;
static xdata char far_countdown = 50;
static xdata enum { sc_none, sc_near, sc_far } sc_type;
static data char treshold;

// the 4 samples needed for the calculations in floating-point
static data struct fp vsamples[4];
static data struct fp isamples[4];
// counter for the 4 samples
static data unsigned char pos;


//-----------------------------------------------------------------------------
// timer2(): Interrupt-handler for sampling and calculation of impedance etc.
//           Function is the only one using floating-point
//           If power is on, the current impedance- and resistance-values are 
//           registered and stored in "R" and "L"
void timer2(void) interrupt 5 using 1 {
  // temporary sample storage
  static xdata struct dac_samplepair tempsamp;
  // indexes for sample-array
  static data char k, k_1, k_2, k__1;
  // temps for calculating
  static data struct fp ik, ik_1, nom;
  
  TF2 = 0;
  
  // real-time clock
  msec+=7;
  if (msec>1999) {
    msec-=2000;
    sec++;
    if (sec>59) {
      sec = 0;
      min++;
      if (min>59) {
        min = 0;
        hour++;
        if (hour>23) {
          hour=0;
          day++;
          day&=63;
        }
      }
    }
  }

  // sample from input
  dac_input( &tempsamp );
  dac_output( &tempsamp );
  // use testsamples instead

  P4_0 = 1;
  // calculate offsets for k into buffer
  pos++; pos &= 0x3;
  k = (pos-2) & 0x3;     // k
  k_1 = (pos-1) & 0x3;   // k+1
  k_2 = pos;             // k+2
  k__1 = (pos-3) & 0x3;  // k-1

  // calculate current sample as floating-point
  if (tempsamp.dac1<0) {
    vsamples[pos].sign = 1;
    tempsamp.dac1 = -tempsamp.dac1;
  } else {
    vsamples[pos].sign = 0;
  }
  vsamples[pos].man = tempsamp.dac1;
  vsamples[pos].exp = BIAS+15;
  if (tempsamp.dac2<0) {
    isamples[pos].sign = 1;
    tempsamp.dac2 = -tempsamp.dac2;
  } else {
    isamples[pos].sign = 0;
  }
  isamples[pos].man = tempsamp.dac2;
  isamples[pos].exp = BIAS+15;
  
  // start by calculating i'[k]
  fp_pushd(&isamples[k_1]);
  fp_pushd(&isamples[k__1]);
  fp_sub();
  fp_popd(&ik);
  // then i'[k+1]
  fp_pushd(&isamples[k_2]);
  fp_pushd(&isamples[k]);
  fp_sub();
  fp_popd(&ik_1);
  
  // now go for R
  fp_pushd(&vsamples[k]);
  fp_pushd(&ik_1);
  fp_mul();
  fp_pushd(&vsamples[k_1]);
  fp_pushd(&ik);
  fp_mul();
  fp_sub();
  fp_pushd(&isamples[k]);
  fp_pushd(&ik_1);
  fp_mul();
  fp_pushd(&isamples[k_1]);
  fp_pushd(&ik);
  fp_mul();
  fp_sub();
  // save the divider for later
  fp_popd(&nom);
  fp_pushd(&nom);
  if (fp_div()) {
    P4_1 = 1;
    fp_popd(&ik);
    fp_popd(&ik);
    R = 0;
    P4_1 = 0;
  } else {
    fp_push(&scale1);
    fp_mul();
    R = fp_popasint();
  }
  
  // and L!!!
  fp_pushd(&vsamples[k_1]);
  fp_pushd(&isamples[k]);
  fp_mul();
  fp_pushd(&vsamples[k]);
  fp_pushd(&isamples[k_1]);
  fp_mul();
  fp_sub();
  fp_pushd(&nom);
  if (fp_div()) {
    fp_popd(&ik);
    fp_popd(&ik);
    L = 0;
  } else {
    fp_push(&scale2);
    fp_mul();
    L = fp_popasint();
  }  
  
  // We really don't need k, k_1, k_2 and k__1 now, so we'll use it for 
  if (power && treshold) treshold--;

  // detect short-circuit
  if (power && !treshold && R>0 && R<=far_limit) {
    if (far_countdown) far_countdown--;
    if (!far_countdown) {
      RL_POWER = 1;
      power = 0;
  	   sc_type = sc_far;
    }      
  	 // if resistance is lower than near_limit then a near short-circuit is detected
  	 if (R<=near_limit) {
      RL_POWER = 1;
      power = 0;
  	   sc_type = sc_near;
    }
  }

  P4_0 = 0;
}
  



void main(void) {
   char xdata ascii[] = { 0,0,0,0, 0,0,0,0, 0,0,0,0 };
   char xdata time[] = "  -  .  .  ";
	char xdata count = 0;
   union intchar xdata interval;

   // Initialise LCD
   lcd_init();
   lcd_writexy(0, 0,"DISTANCE RELAY 1.0");
   lcd_writexy(0, 1,"----------------------------------------");
   lcd_writexy(0, 2,"Using 3.5 ms per sample");
   lcd_writexy(0, 5,"R(Ohm) = ");
   lcd_writexy(0, 6,"L(mH) = ");
   lcd_writexy(0,13,"(A) Power relay:        off");
   lcd_writexy(0,14,"(C) Near short-circuit: off");
   lcd_writexy(0,15,"(F) Far short-circuit:  off");
   
   // Make sure relays are ready for use
   RL_POWER = 1; power = 0;
   RL_NEAR = 1; near = 0;
   RL_FAR = 1; far = 0;

   // Initialise ADC
   dac_init();

	// start keyboard
   kb_init();

   // setup timer2 for sampling
   T2CON |= 0x11;        // timer control word: 0 xx 10 0 01 - no prescaler, reload on overflow, comparemode 0, 
   interval.i = -3500;   // interval for timer2 interrupt
   CRCH = interval.c[0]; //  Reload-vrdi for timer2
   CRCL = interval.c[1]; // -+
	CTCON &= ~0x80;       // T2PS1=0 prescaler level 1 must be zero

   // setup interrupts subsystem   
   IP1 = 0x22;  // set priorities of timer0=2, timer1=3
   IP0 = 0x20;
   IEN1 = 0;    // disable external interrupts
   IEN0 = 0xA2; // allow timer0 and timer2 interrupt, allow all interrupts
   
   // Proceed forever
   while (1) {
     while (msec<1000 && !kb_hit() && sc_type==sc_none) ;
     time[0] = (day/10)+'0';  time[ 1] = (day%10)+'0';
     time[3] = (hour/10)+'0'; time[ 4] = (hour%10)+'0';
     time[6] = (min/10)+'0';  time[ 7] = (min%10)+'0';
     time[9] = (sec/10)+'0';  time[10] = (sec%10)+'0';
     lcd_writexy(29,0,time);
     lcd_writexy(24,13,(power?"on ":"off"));
     lcd_writexy(24,14,(near?"on ":"off")); 
     lcd_writexy(24,15,(far?"on ":"off")); 
     
     if (power) {
       int2asc( R, ascii );
       lcd_writexy(10,5, ascii);
       int2asc( L, ascii );
       lcd_writexy(10,6, ascii);
     }

     // empty keyboard-queue
     while (kb_hit()) {
       switch (kb_readkey()) {
         case 'A' :  // all breakers
           far_countdown = 50;
           treshold = 8;
           power = !power;
           RL_POWER = !power;
           lcd_writexy(24,13,(power?"on ":"off"));
           lcd_writexy(2,8,"                          "); 
           break; 
           
         case 'C' : // close breaker
           near = !near; 
           RL_NEAR = !near; 
           lcd_writexy(24,14,(near?"on ":"off")); 
           break; 
           
         case 'F' : // far breaker
           far = !far; 
           RL_FAR = !far; 
           lcd_writexy(24,15,(far?"on ":"off")); 
           break; 
         
         case '0': // sample
           Rr = R; 
           Ll = L;
           int2asc( Rr, ascii );
           lcd_writexy(20,5, ascii);
           int2asc( Ll, ascii );
           lcd_writexy(20,6, ascii);
           break;
           
       }
     }
     
     if (sc_type!=sc_none) {
       switch (sc_type) {
       	case sc_near: lcd_writexy(2,8,"!!! NEAR SHORT CIRCUIT !!!"); break;
       	case sc_far: lcd_writexy(2,8,"!!! FAR SHORT CIRCUIT !!!"); break;
       }
       sc_type = sc_none;
     }       
   }	//end while

}

