#ifndef KEYBOARD_H
#define KEYBOARD_H

#include "general.h"

// KEYBOARD DRIVER
//
// The keyboard is a passive unit polled by the keyboard driver.
// The keyboard driver installs a keyboard scanner on Timer0, that scans the 
// keyboard ~n times per second and adds detected keypresses to the internal
// queue.
// The keyboard-queue has a size of 7 presses. 2 simultanous keypresses 
// results in two adds to the queue. 
// The scanner will not detect the keypress in its first press but wait 
// for the second pass to detect it. This is done by keeping a keyboard-
// map of all presently pressed keys in memory (16 bytes) where the count
// of updates where the key is detected are stored...
//
// WARNING: THE INTERRUPT-DRIVEN KEYBOARD SCANNER USES REGISTER BANK 2 !!!
//          AND TIMER-0
//
// Using timer0 requires special attention since timer0 does not have an
// auto-reload feature - this means that ticks are lost when using timer0
// in mode 01 (16-bit timer). The problem is that if we need an auto-reload
// on timer0, we must use mode 10, but in 10 only an 8-bit timer is available!
// Therefore we choose to use Timer0 in mode 01 (16-bit timer) and just manually
// reload the timer; we loose ticks, but who cares - its not a time-critical 
// task we perform in keyboard scanning - it also allow us to use smaller 
// intervals when a key is detected. The default time is 20 refreshes per
// second (can't get much lower), allowing pretty high keyboard speeds.
//
//
//
// RESOURCES USED:
// ---------------
// Max calling depth: 2, 1 byte parameter
// Memory: 10 bytes on queue (can be reduced to 9)
//

// Initialise the keyboard driver
void kb_init(void);

// Reads single character from keyboard-queue
// Returns value in range 0..15 (corresponding to keys 0 through 9 and A through F)
// May return 0xFF if no keypress is present in keyboard-queue
char kb_readkey(void);

// Is a keypress present in the queue
char kb_hit(void);



#endif
